from setuptools import setup

setup(name='nonsqlite',
      version='0.6.4',
      description='Non SQL Database',
      url='https://github.com/emilianobilli/nonsqlite',
      author='Emiliano Billi',
      author_email='emiliano.billi@gmail.com',
      license='GPL',
      packages=['nonsqlite'],
      zip_safe=False)